from .client import RestClient
from .recognition import RecognitionClient
from .tagging import GenericTaggingClient, FashionTaggingClient
from .colors import DominantColorProductClient, DominantColorGenericClient
from .smart import SmartSearchClient
from .detection import DetectionClient
from .search import SimilarityPhotosClient, SimilarityProductsClient
